function Storage() {
	this.getValue = function(pKey) {
		debug("getValue: " + pKey);
		return window.localStorage.getItem(pKey);
	};

	this.setValue = function(pKey, pValue) {
		if (typeof pValue === "object") {
			pValue = JSON.stringify(pValue);
			// JSON.parse
		}
		// debug("setValue: " + pKey + " : " + pValue);
		return window.localStorage.setItem(pKey, pValue);
	};

	this.clear = function() {
		return window.localStorage.clear();
	};
}

function Notifications() {
	var self = this;
	var notificationsToAdd = null;

	this.addNotificationList = function(pNotifications) {
		debug(pNotifications);
		notificationsToAdd = {};
		if ($.type(pNotifications) == "array") {
			debug("window.plugin.notification.local.add ... ");
			$.each(pNotifications, function(i, params) {
				notificationsToAdd[params.id] = false;
				window.plugin.notification.local.add(params);
			});
			debug("window.plugin.notification.local.add ... end");
		}
	}

	this.init = function() {
		window.plugin.notification.local.onclick = function(id, state, json) {
			json = JSON.parse(json);
			if (json.uri != undefined) {
				app.setStartUrl(json.uri);
			}
		};

		window.plugin.notification.local.onadd = function(id, state, json) {
			if (notificationsToAdd[id] != undefined) {
				notificationsToAdd[id] = true;
				var allAdded = true;
				$.each(notificationsToAdd, function(i, added) {
					if (added == false) {
						allAdded = false;
						return false;
					}
				});
				if (allAdded) {
					debug("ALL ADDED.... call iframe ... ");
					app.communication.sendMessage("addNotificationListDone",
							notificationsToAdd);
				}
			} else {
				debug('notificationsToAdd[id] == undefined!!');
			}
		};
	}
}

function Communication() {
	var self = this;

	this.getMessage = function(event) {
		debug("getMessage");
		try {
			// debug(JSON.stringify(event.data));
			if (event.data["getItem"] != undefined) {
				var key = event.data["getItem"];
				var value = app.storage.getValue(key);
				self.sendMessage(key, value);
			} else if (event.data["setItem"] != undefined) {
				var itemKey = event.data["setItem"]["key"];
				var itemValue = event.data["setItem"]["value"];
				if (itemKey != undefined && itemValue != undefined) {
					app.storage.setValue(itemKey, itemValue);
				} else {
					debug("item key or value not set: "
							+ JSON.stringify(event.data));
				}
			} else if (event.data["clearLocalStorage"]) {
				window.plugin.notification.local.cancelAll();
				app.storage.clear();
			} else if (event.data["addNotificationList"]) {
				app.notifications
						.addNotificationList(event.data["addNotificationList"]);
			} else if (event.data["call"] != undefined) {
				var object = eval(event.data["call"]["object"]);
				var functionName = event.data["call"]["function"];
				if (typeof object == "object"
						&& typeof object[functionName] == "function") {

					debug("call functionName ... "+functionName);

					var params = null;
					if (event.data["call"]["callbackfunction"]) {
						params = {
							"data" : event.data["call"],
							"args" : ""
						};

						object[functionName].apply(object, [ function() {
							debug("callbackfunction: "+ functionName);
							var args = arguments;
							if (Array.prototype.slice) {
								args = Array.prototype.slice.call(arguments);
							}
							params["args"] = JSON.stringify(args);
							self.sendMessage("callbackfunction", params);
						} ]);
					} else {
						params = event.data["call"]["params"];
						if ($.type(params) == "string") {
							params = JSON.parse(params);
						}
						params = ($.type(params) == "array") ? params
								: [ params ];

						object[functionName].apply(object, params);
					}
				}
			} else if (event.data["setIframeUri"] != undefined) {
				app.setIframeUri(event.data["setIframeUri"]);
			} else if (event.data["openWindow"] != undefined) {
				debug("getMessage - openWindow");
				var strUrl = event.data["openWindow"]["strUrl"] || "";
				var strWindowName = event.data["openWindow"]["strWindowName"]
						|| "";
				var strWindowFeatures = event.data["openWindow"]["strWindowFeatures"]
						|| "";
				app.openWindow(strUrl, strWindowName, strWindowFeatures);
			} else if (event.data["getVersion"] != undefined) {
				self.sendMessage("version", app.version);
			}

			if (event.error) {
				debug("Error! " + event.error);
			}
		} catch (e) {
			debug(e);
		}
	};

	this.sendMessage = function(pKey, pObject) {
		// debug("sendMessage");
		try {
			app.iframe.contentWindow.postMessage({
				"key" : pKey,
				"value" : pObject
			}, app.rootUrl);
		} catch (e) {
			debug(e);
		}
	};

	this.callbackfunction = function(args) {
		debug("callbackfunction ... ");
		debug(args);
		debug(arguments);
	}

	this.init = function() {
		window.addEventListener("message", self.getMessage, false);
	}
}

var app = null;
function App() {
	var self = this;
	this.version = "1.2.0";

	this.rootUrl = "http://www.gesund-ohne-diabetes.de";

	// this.rootUrl = "http://192.168.137.130";
	// this.rootUrl = "http://192.168.137.131";
	// this.rootUrl = "http://diabetes.betaproxy.cellms.de";
	// this.rootUrl = "http://diabetes.liveproxy.cellms.de";

	this.iframe = document.getElementById('iframe');
	this.storage = new Storage();
	this.communication = new Communication();
	this.notifications = new Notifications();

	var loadTimeout = null;
	var startUrlSet = false;
	var startUrl = null;

	this.options = {
		"ajaxTimeout" : 30000,
		"reconnectTimeout" : 60000,
		"reconnectErrorAlertTitle" : "Verbindung fehlgeschlagen",
		"reconnectErrorAlertMessage" : "Der Inhalt konnte nicht geladen werden.",
		"reconnectErrorAlertButtonName" : "Erneut verbinden"

	}

	window.deviceInfo = {
		"phonegap" : true
	};
	function bindEvents() {
		// Bind any events that are required on startup. Common events are:
		// 'load', 'deviceready', 'offline', and 'online'.
		document.addEventListener('deviceready', self.onDeviceReady, false);
	}

	this.connectionError = function() {
		debug("connectionError");
		$("#load").show();
		$("#iframeWrapper").hide();

		// TODO check iframe state

		function reconnect() {
			debug("reconnect ... ");
			if (!!$(self.iframe).attr("data-src")) {
				debug('$(self.iframe).attr("data-src") ... ');
				loadTimeout = window.setTimeout(function() {
					debug("reconnect::loadTimeout ... connectionError")
					self.connectionError();
				}, self.options.reconnectTimeout);
				start($(self.iframe).attr("data-src"));
			} else {
				self.getStartUrl();
			}
		}

		try {
			navigator.notification.alert(
					self.options.reconnectErrorAlertMessage,
					// 'Die Seite konnte nicht geladen werden.', // message
					reconnect, // callback
					self.options.reconnectErrorAlertTitle, // title
					// 'Verbindung fehlgeschlagen', // title
					self.options.reconnectErrorAlertButtonName // buttonName
			// 'Erneut verbinden' // buttonName
			);
		} catch (e) {
			debug("notification exception");
			alert(self.options.reconnectErrorAlertMessage);
			setTimeout(reconnect, 1);
		}
	}

	this.getStartUrl = function() {
		var data = {
			"deviceInfo" : window.deviceInfo,
			"device" : app.storage.getValue("device"),
			"app" : {
				"version" : self.version,
				"startCount" : self.storage.getValue("startCount")
			}
		};
		debug("getStartUrl:" + self.rootUrl);
		debug(data);
		$.ajax({
			type : "POST",
			dataType : "json",
			url : self.rootUrl + "/api/app/action.start/",
			data : data,
			timeout : self.options.ajaxTimeout
		}).done(function(response) {
			debug("getStartUrl:success");
			debug(response);
			if (response && response.data && response.data.startUrl) {
				self.storage.setValue('start', response.data);
				debug("startUrlSet: " + startUrlSet);
				if (!startUrlSet) {
					startUrl = response.data.startUrl;
					start(response.data.startUrl);
				}
				if (response.data.options) {
					$.extend(true, self.options, response.data.options);
					self.storage.setValue("options", self.options);
				}
			}
		}).fail(function(xhr, ajaxOptions, thrownError) {
			debug("ajax-error: ");
			debug("xhr.status: " + xhr.status);
			debug(ajaxOptions);
			debug(thrownError);
			self.connectionError();
		});
	}

	this.setIframeUri = function(pUrl) {
		debug("pUrl: " + pUrl);
		debug("self.iframe.src: " + self.iframe.src);

		loadTimeout = window.setTimeout(function() {
			debug("loadTimeout ... connectionError")
			self.connectionError();
		}, self.options.reconnectTimeout);

		$(self.iframe).attr("data-src", self.rootUrl + pUrl);
	}

	this.openWindow = function(strUrl, strWindowName, strWindowFeatures) {
		debug("diabetes.js - openWindow");
		debug(arguments);
		// debug("diabetes.js - openWindow - window.open");
		// window.open.apply(window, arguments);
		debug("diabetes.js - openWindow - loadUrl");
		navigator.app.loadUrl(strUrl, {
			openExternal : true
		});
		// debug("diabetes.js - openWindow - location.href");
		// window.location.href = strUrl;
	}

	function start(pUrl) {
		debug("start:" + pUrl);
		debug("self.iframe.src:" + self.iframe.src);
		if (pUrl) {
			// do an ajax call to get the right starturl
			self.iframe.src = pUrl;
		}
	}
	this.setStartUrl = function(pUrl) {
		debug("start url set ... ");
		startUrlSet = true;
		$(self.iframe).attr("data-src", self.rootUrl + pUrl);
		start(self.rootUrl + pUrl);
	}

	this.init = function() {
		debug("init");
		bindEvents();
		$(document).ready(function() {
			debug("documentReady");

			$(self.iframe).on("load", function(event) {
				debug("iframe load ... ");
				var iframeContent = $(self.iframe).contents();
				var meta = iframeContent.find('[name*="diabetes"]');
				if (meta.length <= 0) {
					self.connectionError();
					if (startUrl && self.iframe.src != startUrl) {
						start(startUrl);
					}
				} else {
					window.clearTimeout(loadTimeout);

					debug("show iframeWrapper");
					$("#iframeWrapper").show();
					$("#load").hide();
				}
			});
		});
	};

	this.onDeviceReady = function() {
		debug("onDeviceReady");
		try {
			var options = self.storage.getValue("options");
			if (options) {
				options = JSON.parse(options);
				if (typeof options == "object") {
					self.options = options;
				}
			}

			// Listen for requests to use it.
			self.communication.init();
			self.notifications.init();

			self.storage.setValue("device", device);
			var startCount = self.storage.getValue("startCount");
			startCount = parseInt(startCount) || 0;
			self.storage.setValue("startCount", startCount + 1);
			self.storage.setValue("deviceInfo", window.deviceInfo);

			self.getStartUrl();

		} catch (e) {
			debug(e);
		}
	};
}

var debugString = "debugString:";
function debug(pValue) {
	return;
	// alert(pValue);
	debugString += "\n" + Date.now() + ":" + pValue;
	if (!!console && !!console.log) {
		console.log(pValue);
	}
	var debugHtml = document.getElementById("debug");
	if (debugHtml) {
		debugHtml.value = debugString;
	}
}

function initApp() {
	debug("initApp");
	app = new App();
	app.init();
}
